@@ -8,8 +8,14 @@
 /*jsl:declare Ext*/
 /*jsl:declare _*/
 
+/**
+ * Maintain three lists, respectively for added, updated and deleted
+ * records, so that the store can give an time ordered list of changes.
+ *
+ * Carry also a list of sticky filters, that survives when the normal
+ * filters are cleared.
+ */
 
-// Maintain two other lists, one for added records, one for deleted ones
 Ext.define("MP.data.Store", {
     extend: "Ext.data.Store",
 
@@ -52,28 +58,52 @@ Ext.define("MP.data.Store", {
         config = Ext.merge({
             listeners: {
                 add: function(store, recs /*, index*/) {
-                    if(!store.added) {
-                        store.added = [];
+                    if(!store._add_recs) {
+                        store._add_recs = [];
                     }
-                    store.added = store.added.concat(recs);
+                    store._add_recs = store._add_recs.concat(recs);
                 },
 
                 remove: function(store, rec /*, index*/) {
-                    if(!store.deleted) {
-                        store.deleted = [];
+                    if(!store._del_recs) {
+                        store._del_recs = [];
+                    }
+                    if(store._del_recs.indexOf(rec) == -1) {
+                        store._del_recs.push(rec);
+                    }
+                    Ext.Array.remove(store._add_recs, rec);
+                    Ext.Array.remove(store._upd_recs, rec);
+                },
+
+                update: function(store, rec, op) {
+                    if(op === Ext.data.Model.REJECT)
+                        return;
+
+                    if(store._add_recs && store._add_recs.indexOf(rec) != -1)
+                        return;
+
+                    if(store._del_recs && store._del_recs.indexOf(rec) != -1)
+                        return;
+
+                    if(!store._upd_recs) {
+                        store._upd_recs = [];
+                    }
+
+                    if(store._upd_recs.indexOf(rec) == -1) {
+                        store._upd_recs.push(rec);
                     }
-                    store.deleted.push(rec);
-                    Ext.Array.remove(store.added, rec);
                 },
 
                 load: function(store /*, recs, options*/) {
-                    store.added = [];
-                    store.deleted = [];
+                    store._add_recs = [];
+                    store._del_recs = [];
+                    store._upd_recs = [];
                 },
 
                 clear: function(store /*, recs, options*/) {
-                    store.added = [];
-                    store.deleted = [];
+                    store._add_recs = [];
+                    store._del_recs = [];
+                    store._upd_recs = [];
                 }
             }
         }, config);
@@ -96,16 +126,23 @@ Ext.define("MP.data.Store", {
         me.addEvents("reject");
     },
 
+    /* Remove the given record from the added, updated or removed lists
+     * of records, if present.
+     */
     removeRecordFromLists: function(record) {
         // update added/deleted lists
-        if(this.added && this.added.indexOf(record) != -1) {
+        if(this._add_recs && this._add_recs.indexOf(record) != -1) {
             // if the record was just added, simply remove it from the
             // added list
-            Ext.Array.remove(this.added, record);
-        } else if(this.deleted && this.deleted.indexOf(record) != -1) {
+            Ext.Array.remove(this._add_recs, record);
+        } else if(this._del_recs && this._del_recs.indexOf(record) != -1) {
             // if the record was deleted, remove it from the deleted
             // list
-            Ext.Array.remove(this.deleted, record);
+            Ext.Array.remove(this._del_recs, record);
+        } else if(this._upd_recs && this._upd_recs.indexOf(record) != -1) {
+            // if the record was changed, remove it from the _upd_recs
+            // list
+            Ext.Array.remove(this._upd_recs, record);
         }
     },
 
@@ -115,7 +152,7 @@ Ext.define("MP.data.Store", {
     },
 
     afterReject: function(record) {
-        var justadded = this.added && this.added.indexOf(record) != -1;
+        var justadded = this._add_recs && this._add_recs.indexOf(record) != -1;
         this.removeRecordFromLists(record);
         if(!justadded) {
             // Don't call afterReject if the record was added (and thus
@@ -125,8 +162,8 @@ Ext.define("MP.data.Store", {
         } else {
             // Remove the stale (just inserted and not confirmed) record
             this.remove(record);
-            if(this.deleted) {
-                Ext.Array.remove(this.deleted, record);
+            if(this._del_recs) {
+                Ext.Array.remove(this._del_recs, record);
             }
         }
     },
@@ -134,13 +171,20 @@ Ext.define("MP.data.Store", {
     rejectChanges: function(suppressEvent) {
         this.callParent();
 
-        var a = this.added.slice(0);
+        var a = this._add_recs.slice(0);
         for(var i = 0, len = a.length; i < len; i++) {
             a[i].reject();
         }
 
-        if(this.deleted && this.deleted.length) {
-            this.deleted = [];
+        if(this._del_recs && this._del_recs.length) {
+            this._del_recs = [];
+            if(!suppressEvent) {
+                this.fireEvent("datachanged", this);
+            }
+        }
+
+        if(this._upd_recs && this._upd_recs.length) {
+            this._upd_recs = [];
             if(!suppressEvent) {
                 this.fireEvent("datachanged", this);
             }
@@ -152,23 +196,46 @@ Ext.define("MP.data.Store", {
     },
 
     getRejectRecords: function() {
-        var recs = this.callParent();
-        if(this.deleted && this.deleted.length>0) {
-            Ext.Array.push(recs, this.deleted);
+        var recs = [], push=Ext.Array.push;
+
+        if(this._add_recs && this._add_recs.length) {
+            push(recs, this._add_recs);
+        }
+        if(this._upd_recs && this._upd_recs.length) {
+            push(recs, this._upd_recs);
+        }
+        if(this._del_recs && this._del_recs.length) {
+            push(recs, this._del_recs);
         }
         return recs;
     },
 
+    getNewRecords: function() {
+        return this._add_recs || [];
+    },
+
+    getUpdatedRecords: function() {
+        return this._upd_recs || [];
+    },
+
+    getDeletedRecords: function() {
+        return this._del_recs || [];
+    },
+
     isModified: function() {
         var modified = false;
         if(this.data.length>0) {
-            modified = (this.getModifiedRecords().length>0 ||
-                        (this.added && this.added.length>0) ||
-                        (this.deleted && this.deleted.length>0));
+            modified = (this._upd_recs && this._upd_recs.length>0 ||
+                        (this._add_recs && this._add_recs.length>0) ||
+                        (this._del_recs && this._del_recs.length>0));
         }
         return modified;
     },
 
+    /**
+     * Compute the set of changes applied to the records in the store,
+     * including added, updated and deleted records.
+     */
     getChanges: function(changes, idfield, idfvalue, deletions_only) {
         var modified;
         var deleted;
@@ -208,7 +275,7 @@ Ext.define("MP.data.Store", {
                 var len = recs.length;
 
                 // first added or modified records
-                add_or_mod = add_or_mod.concat(this.added || []);
+                add_or_mod = add_or_mod.concat(this._add_recs || []);
                 for(var aomi=0; aomi<len; aomi++) {
                     var rec = recs[aomi];
                     if(!rec.isValid()) {
@@ -266,7 +333,7 @@ Ext.define("MP.data.Store", {
             }
 
             // finally deleted records
-            var drds = this.deleted || [];
+            var drds = this._del_recs || [];
             for(var di=0; di < drds.length; di++) {
                 deleted.push([idfield, drds[di].get(idfield)]);
             }
@@ -279,6 +346,9 @@ Ext.define("MP.data.Store", {
         }
     },
 
+    /**
+     * Commit the changes, if any, with a ``POST`` Ajax request to the given url.
+     */
     commitChanges: function(url, idfield, callback, scope, deletions_only) {
         var me = this;
 
@@ -382,27 +452,34 @@ Ext.define("MP.data.Store", {
         });
     },
 
+    /**
+     * Determine the CSS class to apply to the given record, accordingly to
+     * its current state within this store.
+     */
     classifyRecord: function(rec) {
-        if(this.deleted && this.deleted.indexOf(rec) != -1) {
+        if(this._del_recs && this._del_recs.indexOf(rec) != -1) {
             return this.deletedRecordCls;
-        } else if (this.added && this.added.indexOf(rec) != -1) {
+        } else if (this._add_recs && this._add_recs.indexOf(rec) != -1) {
             return this.newRecordCls;
-        } else if (this.getUpdatedRecords().indexOf(rec) != -1) {
+        } else if (this._upd_recs && this._upd_recs.indexOf(rec) != -1) {
             return this.dirtyRecordCls;
         }
         return '';
     },
 
     deleteRecord: function(rec) {
-        if(!this.deleted) {
-            this.deleted = [];
+        if(!this._del_recs) {
+            this._del_recs = [];
         }
-        if(this.added.indexOf(rec) != -1) {
+        if(this._add_recs.indexOf(rec) != -1) {
             // It's a new record, remove it immediately
             this.remove(rec);
-            Ext.Array.remove(this.deleted, rec);
-        } else if(this.deleted.indexOf(rec) == -1) {
-            this.deleted.push(rec);
+            Ext.Array.remove(this._del_recs, rec);
+        } else if(this._del_recs.indexOf(rec) == -1) {
+            this._del_recs.push(rec);
+            if (this._upd_recs) {
+                Ext.Array.remove(this._upd_recs, rec);
+            }
             this.fireEvent("update", this, rec, Ext.data.Record.EDIT, []);
             this.fireEvent("datachanged", this);
         }
@@ -449,9 +526,13 @@ Ext.define("MP.data.Store", {
      * by the server, plus eventual additions.
      */
     getTotalCount : function() {
-        return (this.totalCount || 0) + (this.added ? this.added.length : 0);
+        return (this.totalCount || 0) + (this._add_recs ? this._add_recs.length : 0);
     },
 
+    /**
+     * Determine whether the given action should be disabled, accordingly
+     * to its configuration and current store state.
+     */
     shouldDisableAction: function(act) {
         if(!act.setDisabled) { return null; }
         var cfg = act.initialConfig;
