@@ -16,6 +16,11 @@ file, for faster load and obfuscation.
 from os.path import abspath, dirname, join, normpath
 from re import compile, DOTALL, MULTILINE
 
+try:
+    FileNotFoundError
+except NameError:
+    FileNotFoundError = IOError
+
 
 DBG_RX = compile(r"^[ \t]*//[ \t]*<debug>[ \t]*$"
                  r".*?"
@@ -190,7 +195,7 @@ def minify_css(args, styles, map):
 
     if args.debug:
         concat_fname = mktemp('.css')
-        concat_output = open(concat_fname, 'w')
+        concat_output = open(concat_fname, 'wb')
 
     for style in styles:
         css = resolve_css_imports(style, map).encode('utf-8')
@@ -216,12 +221,12 @@ def minify_css(args, styles, map):
         unlink(concat_fname)
 
 
-def absolutize(paths, map):
+def absolutize(paths, prefix_map):
     """Transform each partial path into its absolute full path.
 
     :param paths: a list of partial paths
-    :param map: a dictionary, where each key is a possible prefix and its value
-                is the absolute position on the filesystem
+    :param prefix_map: a dictionary, where each key is a possible prefix and its value
+                       is the absolute position on the filesystem
     :rtype: a list of absolute paths
 
     Example::
@@ -234,10 +239,13 @@ def absolutize(paths, map):
 
     abspaths = []
     for path in paths:
-        for prefix, base in map.items():
-            if path.startswith(prefix):
-                abspaths.append(join(base, path[len(prefix):]))
-                break
+        if '/' in path:
+            for prefix, base in prefix_map.items():
+                if path.startswith(prefix):
+                    abspaths.append(join(base, path[len(prefix):]))
+                    break
+            else:
+                abspaths.append(path)
         else:
             abspaths.append(path)
     return abspaths
@@ -274,13 +282,21 @@ def main():
     parser.add_argument('--prefix-map', action='append', default=[],
                         help="Specify a map between a prefix and a"
                         " destination path")
+    parser.add_argument('--extjs-auto-deps', default=False, action="store_true",
+                        help="Automatically extract ExtJS dependencies")
+    parser.add_argument('--extjs-core-bundle',
+                        help="Assume core classes are loaded from the"
+                        " specified bundle")
+    parser.add_argument('--extjs-require', action='append', default=[],
+                        help="Specify a required ExtJS script or class")
+
     args = parser.parse_args()
 
     prefix_map = dict()
     for item in args.prefix_map:
         prefix, base = item.split('=')
         prefix = prefix.strip()
-        if not prefix.endswith('/'):
+        if '/' in prefix and not prefix.endswith('/'):
             prefix += '/'
         base = base.strip()
         if not base.endswith('/'):
@@ -288,10 +304,15 @@ def main():
         prefix_map[prefix] = base
 
     with open(args.manifest) as f:
-        sources = absolutize(load(f), prefix_map)
+        sources = load(f)
 
-    scripts = [script for script in sources if script.endswith('.js')]
-    styles = [script for script in sources if script.endswith('.css')]
+    styles = absolutize(sources.get('styles', []), prefix_map)
+    scripts = absolutize(sources.get('scripts', []), prefix_map)
+
+    if args.extjs_auto_deps:
+        from .extjs import get_needed_sources
+        scripts = get_needed_sources(args.extjs_require + scripts, prefix_map,
+                                     bundle=args.extjs_core_bundle)
 
     if args.list_js or args.list_css:
         if args.list_css:
